#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Indicator;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Strategy;
#endregion

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    /// <summary>
    /// Based off of the TASC February 2014 article, The Degree of Complexity, written by Oscar G. Cagigas
    /// </summary>
    [Description("Based off of the TASC February 2014 article, The Degree of Complexity, written by Oscar G. Cagigas")]
    public class DonchianTradingSystem : Strategy
    {
        #region Variables
		
        //User defined inputs
        public int len1 = 40;
        public int len2 = 15;
		public int aTRStop = 4;
		public double k = 0.9;
		
		
		
		//Non UDI
		private double BuyStop;
		private double SellStop;
		private double ShortStop;
		private double CoverStop;
		private double BuyPrice;
		private double ShortPrice;
		private double CoverPrice;
		private double SellPrice;
	
        
        #endregion

        /// <summary>
        /// This method is used to configure the strategy and is called once before any strategy method is called.
        /// </summary>
        protected override void Initialize()
        {
			Add(TASCDonchian());
			
			AccountSize = 100000;
			
			EntriesPerDirection = 4;
			
			Add(PeriodType.Day, 1);
			
            CalculateOnBarClose = true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if(CurrentBars[0] < 1 || CurrentBars[1] < 1)
				return; 
			
			//BuyStop,SellStop
			BuyStop = this.TASCDonchian().LongUpper[1];
			SellStop = this.TASCDonchian().ShortLower[1];
			ShortStop = this.TASCDonchian().LongLower[1];
			CoverStop = this.TASCDonchian().ShortUpper[1];
			
			//Order Entry
			//Long
			if(BarsInProgress == 0)
			{
				if(High[0] > BuyStop) 
				{					
					BuyPrice = Math.Max(BuyStop, Open[0]);
					EnterLongStop(BuyPrice, "Buy Stop");				
				}
				
				//Short
				else if(Low[0] < ShortStop)
				{
					ShortPrice = Math.Min(ShortStop, Open[0]);
					EnterShortStop(ShortPrice, "Short Stop");		
				}
				
				//Exit Orders
				if(Position.MarketPosition != MarketPosition.Flat)
				{
					//Sell
					if(Position.MarketPosition == MarketPosition.Long)
					{
						if(Low[0] < SellStop)
						{
							SellPrice = BuyPrice - ATR(4)[0];
							ExitLongStop(SellPrice, "Buy Stop");
						}
					}
					
					//Cover
					else if(Position.MarketPosition == MarketPosition.Short)
					{
						if(High[0] > CoverStop)
						{
							CoverPrice = ShortPrice + ATR(4)[0];
							ExitShortStop(CoverPrice, "Short Stop");
						}
					}
				}
			}
		}
		
	

        #region Properties
        [Description("")]
        [GridCategory("Parameters")]
        public int Len1
        {
            get { return len1; }
            set { len1 = Math.Max(1, value); }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int Len2
        {
            get { return len2; }
            set { len2 = Math.Max(1, value); }
        }
		
		[Description("")]
        [GridCategory("Parameters")]
        public int ATRStop
        {
            get { return aTRStop; }
            set { aTRStop = Math.Max(1, value); }
        }
		
		[Description("")]
        [GridCategory("Parameters")]
        public double K
        {
            get { return k; }
            set { k = Math.Max(1, value); }
        }
        #endregion
    }
}
